/*
* Licensed to the Apache Software Foundation (ASF) under one or more
* contributor license agreements.  See the NOTICE file distributed with
* this work for additional information regarding copyright ownership.
* The ASF licenses this file to You under the Apache License, Version 2.0
* (the "License"); you may not use this file except in compliance with
* the License.  You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

    package com.ywt.gapi.mq;

import org.apache.dubbo.common.stream.StreamObserver;
import org.apache.dubbo.common.URL;
import org.apache.dubbo.rpc.Invoker;
import org.apache.dubbo.rpc.PathResolver;
import org.apache.dubbo.rpc.RpcException;
import org.apache.dubbo.rpc.ServerService;
import org.apache.dubbo.rpc.TriRpcStatus;
import org.apache.dubbo.rpc.model.MethodDescriptor;
import org.apache.dubbo.rpc.model.ServiceDescriptor;
import org.apache.dubbo.rpc.model.StubMethodDescriptor;
import org.apache.dubbo.rpc.model.StubServiceDescriptor;
import org.apache.dubbo.rpc.stub.BiStreamMethodHandler;
import org.apache.dubbo.rpc.stub.ServerStreamMethodHandler;
import org.apache.dubbo.rpc.stub.StubInvocationUtil;
import org.apache.dubbo.rpc.stub.StubInvoker;
import org.apache.dubbo.rpc.stub.StubMethodHandler;
import org.apache.dubbo.rpc.stub.StubSuppliers;
import org.apache.dubbo.rpc.stub.UnaryStubMethodHandler;

import com.google.protobuf.Message;

import java.util.HashMap;
import java.util.Map;
import java.util.function.BiConsumer;
import java.util.concurrent.CompletableFuture;

public final class DubboMQExecutorServiceTriple {

    public static final String SERVICE_NAME = MQExecutorService.SERVICE_NAME;

    private static final StubServiceDescriptor serviceDescriptor = new StubServiceDescriptor(SERVICE_NAME,MQExecutorService.class);

    static {
        org.apache.dubbo.rpc.protocol.tri.service.SchemaDescriptorRegistry.addSchemaDescriptor(SERVICE_NAME,MqProto.getDescriptor());
        StubSuppliers.addSupplier(SERVICE_NAME, DubboMQExecutorServiceTriple::newStub);
        StubSuppliers.addSupplier(MQExecutorService.JAVA_SERVICE_NAME,  DubboMQExecutorServiceTriple::newStub);
        StubSuppliers.addDescriptor(SERVICE_NAME, serviceDescriptor);
        StubSuppliers.addDescriptor(MQExecutorService.JAVA_SERVICE_NAME, serviceDescriptor);
    }

    @SuppressWarnings("all")
    public static MQExecutorService newStub(Invoker<?> invoker) {
        return new MQExecutorServiceStub((Invoker<MQExecutorService>)invoker);
    }

    /**
         * <pre>
         * gRPC方法名(/${package}.${service}/${method})，如：/com.ywt.gapi.mq.MQExecutorService/executeGRPC
         * </pre>
         */
    private static final StubMethodDescriptor executeGRPCMethod = new StubMethodDescriptor("executeGRPC",
    com.ywt.gapi.mq.ExecuteGRPCRequest.class, com.ywt.gapi.mq.ExecuteGRPCResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.mq.ExecuteGRPCRequest::parseFrom,
    com.ywt.gapi.mq.ExecuteGRPCResponse::parseFrom);

    private static final StubMethodDescriptor executeGRPCAsyncMethod = new StubMethodDescriptor("executeGRPC",
    com.ywt.gapi.mq.ExecuteGRPCRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.mq.ExecuteGRPCRequest::parseFrom,
    com.ywt.gapi.mq.ExecuteGRPCResponse::parseFrom);

    private static final StubMethodDescriptor executeGRPCProxyAsyncMethod = new StubMethodDescriptor("executeGRPCAsync",
    com.ywt.gapi.mq.ExecuteGRPCRequest.class, com.ywt.gapi.mq.ExecuteGRPCResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.mq.ExecuteGRPCRequest::parseFrom,
    com.ywt.gapi.mq.ExecuteGRPCResponse::parseFrom);





    public static class MQExecutorServiceStub implements MQExecutorService{
        private final Invoker<MQExecutorService> invoker;

        public MQExecutorServiceStub(Invoker<MQExecutorService> invoker) {
            this.invoker = invoker;
        }

            /**
         * <pre>
         * gRPC方法名(/${package}.${service}/${method})，如：/com.ywt.gapi.mq.MQExecutorService/executeGRPC
         * </pre>
         */
        @Override
        public com.ywt.gapi.mq.ExecuteGRPCResponse executeGRPC(com.ywt.gapi.mq.ExecuteGRPCRequest request){
            return StubInvocationUtil.unaryCall(invoker, executeGRPCMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.mq.ExecuteGRPCResponse> executeGRPCAsync(com.ywt.gapi.mq.ExecuteGRPCRequest request){
            return StubInvocationUtil.unaryCall(invoker, executeGRPCAsyncMethod, request);
        }

            /**
         * <pre>
         * gRPC方法名(/${package}.${service}/${method})，如：/com.ywt.gapi.mq.MQExecutorService/executeGRPC
         * </pre>
         */
        @Override
        public void executeGRPC(com.ywt.gapi.mq.ExecuteGRPCRequest request, StreamObserver<com.ywt.gapi.mq.ExecuteGRPCResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, executeGRPCMethod , request, responseObserver);
        }



    }

    public static abstract class MQExecutorServiceImplBase implements MQExecutorService, ServerService<MQExecutorService> {

        private <T, R> BiConsumer<T, StreamObserver<R>> syncToAsync(java.util.function.Function<T, R> syncFun) {
            return new BiConsumer<T, StreamObserver<R>>() {
                @Override
                public void accept(T t, StreamObserver<R> observer) {
                    try {
                        R ret = syncFun.apply(t);
                        observer.onNext(ret);
                        observer.onCompleted();
                    } catch (Throwable e) {
                        observer.onError(e);
                    }
                }
            };
        }

        @Override
        public final Invoker<MQExecutorService> getInvoker(URL url) {
            PathResolver pathResolver = url.getOrDefaultFrameworkModel()
            .getExtensionLoader(PathResolver.class)
            .getDefaultExtension();
            Map<String,StubMethodHandler<?, ?>> handlers = new HashMap<>();

            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/executeGRPC" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/executeGRPCAsync" );

            BiConsumer<com.ywt.gapi.mq.ExecuteGRPCRequest, StreamObserver<com.ywt.gapi.mq.ExecuteGRPCResponse>> executeGRPCFunc = this::executeGRPC;
            handlers.put(executeGRPCMethod.getMethodName(), new UnaryStubMethodHandler<>(executeGRPCFunc));
            BiConsumer<com.ywt.gapi.mq.ExecuteGRPCRequest, StreamObserver<com.ywt.gapi.mq.ExecuteGRPCResponse>> executeGRPCAsyncFunc = syncToAsync(this::executeGRPC);
            handlers.put(executeGRPCProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(executeGRPCAsyncFunc));




            return new StubInvoker<>(this, url, MQExecutorService.class, handlers);
        }


        @Override
        public com.ywt.gapi.mq.ExecuteGRPCResponse executeGRPC(com.ywt.gapi.mq.ExecuteGRPCRequest request){
            throw unimplementedMethodException(executeGRPCMethod);
        }





        @Override
        public final ServiceDescriptor getServiceDescriptor() {
            return serviceDescriptor;
        }
        private RpcException unimplementedMethodException(StubMethodDescriptor methodDescriptor) {
            return TriRpcStatus.UNIMPLEMENTED.withDescription(String.format("Method %s is unimplemented",
                "/" + serviceDescriptor.getInterfaceName() + "/" + methodDescriptor.getMethodName())).asException();
        }
    }

}
