package com.ywt.controllers.mg;


import com.ywt.mg.configs.DomainConfigurer;
import com.ywt.mg.core.MGRight;
import com.ywt.mg.core.MGRightTypeDef;
import com.ywt.mg.core.PagedList;
import com.ywt.mg.core.utils.*;
import com.ywt.mg.domain.entities.Goods;
import com.ywt.mg.params.mg.goods.GoodIdRequest;
import com.ywt.mg.params.mg.goods.GoodsVoRequest;
import com.ywt.mg.params.mg.goods.QueryGoodsListRequest;
import com.ywt.mg.services.GoodsService;
import com.ywt.mg.services.UserService;
import com.ywt.mg.web.BaseResponse;
import com.ywt.mg.web.DataResponse;
import io.swagger.annotations.Api;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController("/mg/goods")
@RequestMapping({"/mg/goods"})
@MGRight
@Api(description = "眼睛店后台商品列表")
public class GoodsController {

    private static Logger logger = LoggerFactory.getLogger(GoodsController.class);

    @DubboReference
    private GoodsService goodsService;

    @DubboReference
    private UserService userService;

    @DubboReference
    private DomainConfigurer domainConfigurer;


    @MGRight(menuCode = {" /queryGoodsList"}, type = MGRightTypeDef.Menu | MGRightTypeDef.Logined)
    @RequestMapping({"/queryGoodsList"})
    BaseResponse queryGoodsList(@RequestBody QueryGoodsListRequest request) {

        DataResponse<Map<String, Object>> response = new DataResponse<>();
        int shopId = Checker.getIntegerValue(userService.getShopId());
        if (shopId <= 0){
            return response.failed(BaseResponse.OTHER_ERROR, "请先联系管理员设置店铺");
        }
        try {
            PagedList<Goods> pagedList = goodsService.queryPageGoodsList(request);
            HashMap<String, Object> map = new HashMap<>();
            if (!Checker.isNone(pagedList.getItems())) {
                List<Map<String, Object>> datas = CollectionUtil.toList(pagedList.getItems(), p -> {
                    Map<String, Object> item = new HashMap<>();
                    item.put("id", Checker.getIntegerValue(p.getId()));
                    item.put("name", Checker.getStringValue(p.getName()));
                    item.put("brandName", Checker.getStringValue(p.getBrandName()));
                    item.put("typeName", Checker.getStringValue(p.getTypeName()));
                    item.put("typeId", Checker.getIntegerValue(p.getTypeId()));
                    item.put("spec", Checker.getStringValue(p.getSpec()));
                    item.put("unitName", Checker.getStringValue(p.getUnitName()));
                    item.put("unitId", Checker.getIntegerValue(p.getUnitId()));
                    String amountStr = FormatUtil.intShrink100ToStr(p.getUnitPrice());
                    item.put("unitPrice", Checker.getStringValue(amountStr));
                    item.put("inventory", Checker.getIntegerValue(p.getInventory()));
                    item.put("pic", Checker.getStringValue(p.getPic()));
                    item.put("weight", Checker.getIntegerValue(p.getWeight()));
                    item.put("status", p.getStatus() == 1 ? "正常" : "禁止");
                    item.put("statusInt", Checker.getIntegerValue(p.getStatus()));
                    String createTime = DateUtil.convertToString(p.getCreateTime());
                    item.put("createTime", Checker.getStringValue(createTime));
                    return item;
                });
                //pageDataResponse.setData(datas);
                map.put("list", datas);

            } else {
                map.put("list", new ArrayList<>());
            }
            map.put("totalCount", pagedList.getTotal());
            map.put("pageIndex", request.getPageIndex());
            map.put("pageSize", request.getPageSize());
            response.setData(map);
            // pageDataResponse.setPageIndex(request.getPageIndex());
            //  pageDataResponse.setPageSize(request.getPageSize());
            // pageDataResponse.setTotalCount(pagedList.getTotal());

        } catch (Exception e) {
            logger.error("/goods/queryGoodsList(): {}", e.getMessage(), e);
            e.printStackTrace();
            return response.failed(BaseResponse.OTHER_ERROR, "数据读取失败");
        }
        return response.succeed("处理成功");
    }

    @MGRight(need = true, menuCode = {"/addOrUpdateGoods"}, type = MGRightTypeDef.Menu | MGRightTypeDef.Logined)
    @RequestMapping({"/addOrUpdateGoods"})
    BaseResponse addOrUpdateGoods(@RequestBody GoodsVoRequest request) {

        BaseResponse response = new BaseResponse();
        if (Checker.isNull(request.getSpec())) {
            return response.failed(BaseResponse.PARAMETER_ERROR, "规格不能为空");
        }
        try {
            int weight = Checker.getIntegerValue(request.getWeight());
            return goodsService.addOrUpdateGoods(request.getId(), request.getName(), request.getBrandName(), request.getTypeId(), request.getSpec(), request.getUnitId(), request.getUnitPrice(), request.getInventory(), weight, request.getPic(), request.getStatus());

        } catch (Exception e) {
            logger.error("/mg/goods/addOrUpdateGoods(): {}", e.getMessage(), e);
            e.printStackTrace();
            return response.failed(BaseResponse.APP_ERROR, "处理失败");
        }
    }


    @MGRight(need = true, menuCode = {"/deletedGoods"}, type = MGRightTypeDef.Menu | MGRightTypeDef.Logined)
    @RequestMapping({"/deletedGoods"})
    BaseResponse deletedGoods(@RequestBody GoodIdRequest request) {
        BaseResponse response = new BaseResponse();
        try {
            return goodsService.deletedGoods(request.getId());
        } catch (Exception e) {
            logger.error("/mg/goods/deletedGoods(): {}", e.getMessage(), e);
            e.printStackTrace();
            return response.failed(BaseResponse.APP_ERROR, "处理失败");

        }

    }


    //下载商品列表
    @MGRight(need = true, menuCode = {"/downloadGoodsList"}, type = MGRightTypeDef.Menu | MGRightTypeDef.Logined)
    @RequestMapping({"/downloadGoodsList"})
    BaseResponse downloadGoodsList(@RequestBody QueryGoodsListRequest request, HttpServletResponse httpServletResponse) {
        DataResponse<Map<String, String>> response = new DataResponse<>();
        Integer shopId = userService.getShopId();
        request.setShopId(shopId);
        String url = domainConfigurer.getYwtGlasses() + "/mg/goods/downloadGoodsListGet";
        Map<String, String> map = new HashMap<>();
        map.put("shopId", shopId.toString());
        String createUrl = BizUtil.createParentUrl(request, url, map);
        Map<String, String> mapper = new HashMap<>();
        mapper.put("url", createUrl);
        response.setData(mapper);
        return response.succeed("成功");
    }

    //下载商品列表2
    @MGRight(need = true, menuCode = {"/downloadGoodsListGet"}, type = MGRightTypeDef.Menu | MGRightTypeDef.Logined)
    @RequestMapping({"/downloadGoodsListGet"})
    void downloadGoodsListGet(QueryGoodsListRequest request, HttpServletResponse httpServletResponse) {

        goodsService.downloadGoods(request, httpServletResponse);

    }

    @MGRight(need = true, menuCode = {"/goodsList"}, type = MGRightTypeDef.Menu | MGRightTypeDef.Logined)
    @RequestMapping({"/goodsList"})
    BaseResponse gooList(@RequestBody QueryGoodsListRequest request) {
        return goodsService.gooodsList(request.getName());
    }

}